@@ -29,15 +29,20 @@
 #include "../include/ctdb_private.h"
 #include <sys/socket.h>
 
-static void daemon_incoming_packet(void *, struct ctdb_req_header *);
+struct ctdb_client_pid_list {
+	struct ctdb_client_pid_list *next, *prev;
+	struct ctdb_context *ctdb;
+	pid_t pid;
+	struct ctdb_client *client;
+};
 
+static void daemon_incoming_packet(void *, struct ctdb_req_header *);
 
 static void print_exit_message(void)
 {
 	DEBUG(DEBUG_NOTICE,("CTDB daemon shutting down\n"));
 }
 
-
 /* called when the "startup" event script has finished */
 static void ctdb_start_transport(struct ctdb_context *ctdb)
 {
@@ -93,6 +98,12 @@ static void block_signal(int signum)
 static int daemon_queue_send(struct ctdb_client *client, struct ctdb_req_header *hdr)
 {
 	client->ctdb->statistics.client_packets_sent++;
+	if (hdr->operation == CTDB_REQ_MESSAGE) {
+		if (ctdb_queue_length(client->queue) > client->ctdb->tunable.max_queue_depth_drop_msg) {
+			DEBUG(DEBUG_ERR,("Drop CTDB_REQ_MESSAGE to client. Queue full.\n"));
+			return 0;
+		}
+	}
 	return ctdb_queue_send(client->queue, (uint8_t *)hdr, hdr->length);
 }
 
@@ -123,7 +134,6 @@ static void daemon_message_handler(struct ctdb_context *ctdb, uint64_t srvid,
 
 	talloc_free(r);
 }
-					   
 
 /*
   this is called when the ctdb daemon received a ctdb request to 
@@ -146,13 +156,6 @@ int daemon_register_message_handler(struct ctdb_context *ctdb, uint32_t client_i
 			 (unsigned long long)srvid));
 	}
 
-	/* this is a hack for Samba - we now know the pid of the Samba client */
-	if ((srvid & 0xFFFFFFFF) == srvid &&
-	    kill(srvid, 0) == 0) {
-		client->pid = srvid;
-		DEBUG(DEBUG_INFO,(__location__ " Registered PID %u for client %u\n",
-			 (unsigned)client->pid, client_id));
-	}
 	return res;
 }
 
@@ -176,6 +179,8 @@ int daemon_deregister_message_handler(struct ctdb_context *ctdb, uint32_t client
 */
 static int ctdb_client_destructor(struct ctdb_client *client)
 {
+	struct ctdb_db_context *ctdb_db;
+
 	ctdb_takeover_client_destructor_hook(client);
 	ctdb_reqid_remove(client->ctdb, client->client_id);
 	if (client->ctdb->statistics.num_clients) {
@@ -186,6 +191,13 @@ static int ctdb_client_destructor(struct ctdb_client *client)
 		DEBUG(DEBUG_ERR,(__location__ " Client disconnecting with %u persistent updates in flight. Starting recovery\n", client->num_persistent_updates));
 		client->ctdb->recovery_mode = CTDB_RECOVERY_ACTIVE;
 	}
+	ctdb_db = find_ctdb_db(client->ctdb, client->db_id);
+	if (ctdb_db) {
+		DEBUG(DEBUG_ERR, (__location__ " client exit while transaction "
+				  "commit active. Forcing recovery.\n"));
+		client->ctdb->recovery_mode = CTDB_RECOVERY_ACTIVE;
+		ctdb_db->transaction_active = false;
+	}
 
 	return 0;
 }
@@ -524,6 +536,17 @@ static void ctdb_daemon_read_cb(uint8_t *data, size_t cnt, void *args)
 	daemon_incoming_packet(client, hdr);
 }
 
+
+static int ctdb_clientpid_destructor(struct ctdb_client_pid_list *client_pid)
+{
+	if (client_pid->ctdb->client_pids != NULL) {
+		DLIST_REMOVE(client_pid->ctdb->client_pids, client_pid);
+	}
+
+	return 0;
+}
+
+
 static void ctdb_accept_client(struct event_context *ev, struct fd_event *fde, 
 			 uint16_t flags, void *private_data)
 {
@@ -532,6 +555,7 @@ static void ctdb_accept_client(struct event_context *ev, struct fd_event *fde,
 	int fd;
 	struct ctdb_context *ctdb = talloc_get_type(private_data, struct ctdb_context);
 	struct ctdb_client *client;
+	struct ctdb_client_pid_list *client_pid;
 #ifdef _AIX
 	struct peercred_struct cr;
 	socklen_t crl = sizeof(struct peercred_struct);
@@ -550,24 +574,41 @@ static void ctdb_accept_client(struct event_context *ev, struct fd_event *fde,
 	set_nonblocking(fd);
 	set_close_on_exec(fd);
 
+	DEBUG(DEBUG_DEBUG,(__location__ " Created SOCKET FD:%d to connected child\n", fd));
+
 	client = talloc_zero(ctdb, struct ctdb_client);
 #ifdef _AIX
 	if (getsockopt(fd, SOL_SOCKET, SO_PEERID, &cr, &crl) == 0) {
 #else
 	if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, &cr, &crl) == 0) {
 #endif
-		talloc_asprintf(client, "struct ctdb_client: pid:%u", (unsigned)cr.pid);
+		DEBUG(DEBUG_INFO,("Connected client with pid:%u\n", (unsigned)cr.pid));
 	}
 
 	client->ctdb = ctdb;
 	client->fd = fd;
 	client->client_id = ctdb_reqid_new(ctdb, client);
-	ctdb->statistics.num_clients++;
+	client->pid = cr.pid;
+
+	client_pid = talloc(client, struct ctdb_client_pid_list);
+	if (client_pid == NULL) {
+		DEBUG(DEBUG_ERR,("Failed to allocate client pid structure\n"));
+		close(fd);
+		talloc_free(client);
+		return;
+	}		
+	client_pid->ctdb   = ctdb;
+	client_pid->pid    = cr.pid;
+	client_pid->client = client;
+
+	DLIST_ADD(ctdb->client_pids, client_pid);
 
 	client->queue = ctdb_queue_setup(ctdb, client, fd, CTDB_DS_ALIGNMENT, 
 					 ctdb_daemon_read_cb, client);
 
 	talloc_set_destructor(client, ctdb_client_destructor);
+	talloc_set_destructor(client_pid, ctdb_clientpid_destructor);
+	ctdb->statistics.num_clients++;
 }
 
 
@@ -617,15 +658,6 @@ failed:
 	return -1;	
 }
 
-/*
-  delete the socket on exit - called on destruction of autofree context
- */
-static int unlink_destructor(const char *name)
-{
-	unlink(name);
-	return 0;
-}
-
 static void sig_child_handler(struct event_context *ev,
 	struct signal_event *se, int signum, int count,
 	void *dont_care, 
@@ -650,7 +682,7 @@ static void sig_child_handler(struct event_context *ev,
 /*
   start the protocol going as a daemon
 */
-int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork)
+int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork, bool use_syslog)
 {
 	int res, ret = -1;
 	struct fd_event *fde;
@@ -683,6 +715,11 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork)
 	}
 	block_signal(SIGPIPE);
 
+	ctdbd_pid = getpid();
+
+
+	DEBUG(DEBUG_ERR, ("Starting CTDBD as pid : %u\n", ctdbd_pid));
+
 	if (ctdb->do_setsched) {
 		/* try to set us up as realtime */
 		ctdb_set_scheduler(ctdb);
@@ -690,7 +727,10 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork)
 
 	/* ensure the socket is deleted on exit of the daemon */
 	domain_socket_name = talloc_strdup(talloc_autofree_context(), ctdb->daemon.name);
-	talloc_set_destructor(domain_socket_name, unlink_destructor);	
+	if (domain_socket_name == NULL) {
+		DEBUG(DEBUG_ALERT,(__location__ " talloc_strdup failed.\n"));
+		exit(12);
+	}
 
 	ctdb->ev = event_context_init(NULL);
 
@@ -730,7 +770,7 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork)
 	}
 
 	/* start frozen, then let the first election sort things out */
-	if (!ctdb_blocking_freeze(ctdb)) {
+	if (ctdb_blocking_freeze(ctdb)) {
 		ctdb_fatal(ctdb, "Failed to get initial freeze\n");
 	}
 
@@ -760,6 +800,14 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork)
 		DEBUG(DEBUG_CRIT,("Failed to set up signal handler for SIGCHLD\n"));
 		exit(1);
 	}
+
+	if (use_syslog) {
+		if (start_syslog_daemon(ctdb)) {
+			DEBUG(DEBUG_CRIT, ("Failed to start syslog daemon\n"));
+			exit(10);
+		}
+	}
+
 	  
 	/* go into a wait loop to allow other nodes to complete */
 	event_loop_wait(ctdb->ev);
@@ -1006,6 +1054,11 @@ int ctdb_daemon_send_message(struct ctdb_context *ctdb, uint32_t pnn,
 	struct ctdb_req_message *r;
 	int len;
 
+	if (ctdb->methods == NULL) {
+		DEBUG(DEBUG_ERR,(__location__ " Failed to send message. Transport is DOWN\n"));
+		return -1;
+	}
+
 	/* see if this is a message to ourselves */
 	if (pnn == ctdb->pnn) {
 		return ctdb_local_message(ctdb, srvid, data);
@@ -1027,3 +1080,144 @@ int ctdb_daemon_send_message(struct ctdb_context *ctdb, uint32_t pnn,
 	return 0;
 }
 
+
+
+struct ctdb_client_notify_list {
+	struct ctdb_client_notify_list *next, *prev;
+	struct ctdb_context *ctdb;
+	uint64_t srvid;
+	TDB_DATA data;
+};
+
+
+static int ctdb_client_notify_destructor(struct ctdb_client_notify_list *nl)
+{
+	int ret;
+
+	DEBUG(DEBUG_ERR,("Sending client notify message for srvid:%llu\n", (unsigned long long)nl->srvid));
+
+	ret = ctdb_daemon_send_message(nl->ctdb, CTDB_BROADCAST_CONNECTED, (unsigned long long)nl->srvid, nl->data);
+	if (ret != 0) {
+		DEBUG(DEBUG_ERR,("Failed to send client notify message\n"));
+	}
+
+	return 0;
+}
+
+int32_t ctdb_control_register_notify(struct ctdb_context *ctdb, uint32_t client_id, TDB_DATA indata)
+{
+	struct ctdb_client_notify_register *notify = (struct ctdb_client_notify_register *)indata.dptr;
+        struct ctdb_client *client = ctdb_reqid_find(ctdb, client_id, struct ctdb_client); 
+	struct ctdb_client_notify_list *nl;
+
+	DEBUG(DEBUG_ERR,("Register srvid %llu for client %d\n", (unsigned long long)notify->srvid, client_id));
+
+	if (indata.dsize < offsetof(struct ctdb_client_notify_register, notify_data)) {
+		DEBUG(DEBUG_ERR,(__location__ " Too little data in control : %d\n", (int)indata.dsize));
+		return -1;
+	}
+
+	if (indata.dsize != (notify->len + offsetof(struct ctdb_client_notify_register, notify_data))) {
+		DEBUG(DEBUG_ERR,(__location__ " Wrong amount of data in control. Got %d, expected %d\n", (int)indata.dsize, (int)(notify->len + offsetof(struct ctdb_client_notify_register, notify_data))));
+		return -1;
+	}
+
+
+        if (client == NULL) {
+                DEBUG(DEBUG_ERR,(__location__ " Could not find client parent structure. You can not send this control to a remote node\n"));
+                return -1;
+        }
+
+	for(nl=client->notify; nl; nl=nl->next) {
+		if (nl->srvid == notify->srvid) {
+			break;
+		}
+	}
+	if (nl != NULL) {
+                DEBUG(DEBUG_ERR,(__location__ " Notification for srvid:%llu already exists for this client\n", (unsigned long long)notify->srvid));
+                return -1;
+        }
+
+	nl = talloc(client, struct ctdb_client_notify_list);
+	CTDB_NO_MEMORY(ctdb, nl);
+	nl->ctdb       = ctdb;
+	nl->srvid      = notify->srvid;
+	nl->data.dsize = notify->len;
+	nl->data.dptr  = talloc_size(nl, nl->data.dsize);
+	CTDB_NO_MEMORY(ctdb, nl->data.dptr);
+	memcpy(nl->data.dptr, notify->notify_data, nl->data.dsize);
+	
+	DLIST_ADD(client->notify, nl);
+	talloc_set_destructor(nl, ctdb_client_notify_destructor);
+
+	return 0;
+}
+
+int32_t ctdb_control_deregister_notify(struct ctdb_context *ctdb, uint32_t client_id, TDB_DATA indata)
+{
+	struct ctdb_client_notify_deregister *notify = (struct ctdb_client_notify_deregister *)indata.dptr;
+        struct ctdb_client *client = ctdb_reqid_find(ctdb, client_id, struct ctdb_client); 
+	struct ctdb_client_notify_list *nl;
+
+	DEBUG(DEBUG_ERR,("Deregister srvid %llu for client %d\n", (unsigned long long)notify->srvid, client_id));
+
+        if (client == NULL) {
+                DEBUG(DEBUG_ERR,(__location__ " Could not find client parent structure. You can not send this control to a remote node\n"));
+                return -1;
+        }
+
+	for(nl=client->notify; nl; nl=nl->next) {
+		if (nl->srvid == notify->srvid) {
+			break;
+		}
+	}
+	if (nl == NULL) {
+                DEBUG(DEBUG_ERR,(__location__ " No notification for srvid:%llu found for this client\n", (unsigned long long)notify->srvid));
+                return -1;
+        }
+
+	DLIST_REMOVE(client->notify, nl);
+	talloc_set_destructor(nl, NULL);
+	talloc_free(nl);
+
+	return 0;
+}
+
+struct ctdb_client *ctdb_find_client_by_pid(struct ctdb_context *ctdb, pid_t pid)
+{
+	struct ctdb_client_pid_list *client_pid;
+
+	for (client_pid = ctdb->client_pids; client_pid; client_pid=client_pid->next) {
+		if (client_pid->pid == pid) {
+			return client_pid->client;
+		}
+	}
+	return NULL;
+}
+
+
+/* This control is used by samba when probing if a process (of a samba daemon)
+   exists on the node.
+   Samba does this when it needs/wants to check if a subrecord in one of the
+   databases is still valied, or if it is stale and can be removed.
+   If the node is in unhealthy or stopped state we just kill of the samba
+   process holding htis sub-record and return to the calling samba that
+   the process does not exist.
+   This allows us to forcefully recall subrecords registered by samba processes
+   on banned and stopped nodes.
+*/
+int32_t ctdb_control_process_exists(struct ctdb_context *ctdb, pid_t pid)
+{
+        struct ctdb_client *client;
+
+	if (ctdb->nodes[ctdb->pnn]->flags & (NODE_FLAGS_BANNED|NODE_FLAGS_STOPPED)) {
+		client = ctdb_find_client_by_pid(ctdb, pid);
+		if (client != NULL) {
+			DEBUG(DEBUG_NOTICE,(__location__ " Killing client with pid:%d on banned/stopped node\n", (int)pid));
+			talloc_free(client);
+		}
+		return -1;
+	}
+
+	return kill(pid, 0);
+}
