@@ -102,6 +102,15 @@ int32_t ctdb_control_trans2_commit(struct ctdb_context *ctdb,
 	struct ctdb_client *client = ctdb_reqid_find(ctdb, c->client_id, struct ctdb_client);
 	struct ctdb_persistent_state *state;
 	int i;
+	struct ctdb_marshall_buffer *m = (struct ctdb_marshall_buffer *)recdata.dptr;
+	struct ctdb_db_context *ctdb_db;
+
+	ctdb_db = find_ctdb_db(ctdb, m->db_id);
+	if (ctdb_db == NULL) {
+		DEBUG(DEBUG_ERR,(__location__ " ctdb_control_trans2_commit: "
+				 "Unknown database db_id[0x%08x]\n", m->db_id));
+		return -1;
+	}
 
 	if (client == NULL) {
 		DEBUG(DEBUG_ERR,(__location__ " can not match persistent_store to a client. Returning error\n"));
@@ -123,18 +132,63 @@ int32_t ctdb_control_trans2_commit(struct ctdb_context *ctdb,
 	*/
 	switch (c->opcode) {
 	case CTDB_CONTROL_PERSISTENT_STORE:
+		if (ctdb_db->transaction_active) {
+			DEBUG(DEBUG_ERR, (__location__ " trans2_commit: a "
+					  "transaction is active on database "
+					  "db_id[0x%08x] - refusing persistent "
+					 " store for client id[0x%08x]\n",
+					  ctdb_db->db_id, client->client_id));
+			return -1;
+		}
 		if (client->num_persistent_updates > 0) {
 			client->num_persistent_updates--;
 		}
 		break;
 	case CTDB_CONTROL_TRANS2_COMMIT:
+		if (ctdb_db->transaction_active) {
+			DEBUG(DEBUG_ERR,(__location__ " trans2_commit: there is"
+					 " already a transaction commit "
+					 "active on db_id[0x%08x] - forbidding "
+					 "client_id[0x%08x] to commit\n",
+					 ctdb_db->db_id, client->client_id));
+			return -1;
+		}
+		if (client->db_id != 0) {
+			DEBUG(DEBUG_ERR,(__location__ " ERROR: trans2_commit: "
+					 "client-db_id[0x%08x] != 0 "
+					 "(client_id[0x%08x])\n",
+					 client->db_id, client->client_id));
+			return -1;
+		}
 		client->num_persistent_updates++;
+		ctdb_db->transaction_active = true;
+		client->db_id = m->db_id;
+		DEBUG(DEBUG_DEBUG, (__location__ " client id[0x%08x] started to"
+				  " commit transaction on db id[0x%08x]\n",
+				  client->client_id, client->db_id));
 		break;
 	case CTDB_CONTROL_TRANS2_COMMIT_RETRY:
 		/* already updated from the first commit */
+		if (client->db_id != m->db_id) {
+			DEBUG(DEBUG_ERR,(__location__ " ERROR: trans2_commit "
+					 "retry: client-db_id[0x%08x] != "
+					 "db_id[0x%08x] (client_id[0x%08x])\n",
+					 client->db_id,
+					 m->db_id, client->client_id));
+			return -1;
+		}
+		DEBUG(DEBUG_DEBUG, (__location__ " client id[0x%08x] started "
+				    "transaction commit retry on "
+				    "db_id[0x%08x]\n",
+				    client->client_id, client->db_id));
 		break;
 	}
 
+	if (ctdb->recovery_mode != CTDB_RECOVERY_NORMAL) {
+		DEBUG(DEBUG_INFO,("rejecting ctdb_control_trans2_commit when recovery active\n"));
+		return -1;
+	}
+
 	state = talloc_zero(ctdb, struct ctdb_persistent_state);
 	CTDB_NO_MEMORY(ctdb, state);
 
@@ -251,7 +305,7 @@ static int ctdb_persistent_store(struct ctdb_persistent_write_state *state)
 		if (ret != 0) {
 			DEBUG(DEBUG_CRIT,("Failed to store record for db_id 0x%08x in ctdb_persistent_store\n", 
 					  state->ctdb_db->db_id));
-			return -1;
+			goto failed;
 		}
 	}
 
@@ -414,8 +468,12 @@ struct childwrite_handle *ctdb_childwrite(struct ctdb_db_context *ctdb_db,
 	}
 
 	close(result->fd[1]);
+	set_close_on_exec(result->fd[0]);
+
 	talloc_set_destructor(result, childwrite_destructor);
 
+	DEBUG(DEBUG_NOTICE, (__location__ " Created PIPE FD:%d for ctdb_childwrite\n", result->fd[0]));
+
 	result->fde = event_add_fd(ctdb_db->ctdb->ev, result, result->fd[0],
 				   EVENT_FD_READ|EVENT_FD_AUTOCLOSE, childwrite_handler,
 				   (void *)result);
@@ -493,6 +551,23 @@ int32_t ctdb_control_trans2_finished(struct ctdb_context *ctdb,
 				     struct ctdb_req_control *c)
 {
 	struct ctdb_client *client = ctdb_reqid_find(ctdb, c->client_id, struct ctdb_client);
+	struct ctdb_db_context *ctdb_db;
+
+	ctdb_db = find_ctdb_db(ctdb, client->db_id);
+	if (ctdb_db == NULL) {
+		DEBUG(DEBUG_ERR,(__location__ " ctdb_control_trans2_finish "
+				 "Unknown database 0x%08x\n", client->db_id));
+		return -1;
+	}
+	if (!ctdb_db->transaction_active) {
+		DEBUG(DEBUG_ERR,(__location__ " ctdb_control_trans2_finish: "
+				 "Database 0x%08x has no transaction commit "
+				 "started\n", client->db_id));
+		return -1;
+	}
+
+	ctdb_db->transaction_active = false;
+	client->db_id = 0;
 
 	if (client->num_persistent_updates == 0) {
 		DEBUG(DEBUG_ERR, (__location__ " ERROR: num_persistent_updates == 0\n"));
@@ -502,6 +577,10 @@ int32_t ctdb_control_trans2_finished(struct ctdb_context *ctdb,
 	}
 	client->num_persistent_updates--;
 
+	DEBUG(DEBUG_DEBUG, (__location__ " client id[0x%08x] finished "
+			    "transaction commit db_id[0x%08x]\n",
+			    client->client_id, ctdb_db->db_id));
+
 	return 0;
 }
 
@@ -513,19 +592,64 @@ int32_t ctdb_control_trans2_error(struct ctdb_context *ctdb,
 				  struct ctdb_req_control *c)
 {
 	struct ctdb_client *client = ctdb_reqid_find(ctdb, c->client_id, struct ctdb_client);
-	
+	struct ctdb_db_context *ctdb_db;
+
+	ctdb_db = find_ctdb_db(ctdb, client->db_id);
+	if (ctdb_db == NULL) {
+		DEBUG(DEBUG_ERR,(__location__ " ctdb_control_trans2_error: "
+				 "Unknown database 0x%08x\n", client->db_id));
+		return -1;
+	}
+	if (!ctdb_db->transaction_active) {
+		DEBUG(DEBUG_ERR,(__location__ " ctdb_control_trans2_error: "
+				 "Database 0x%08x has no transaction commit "
+				 "started\n", client->db_id));
+		return -1;
+	}
+
+	ctdb_db->transaction_active = false;
+	client->db_id = 0;
+
 	if (client->num_persistent_updates == 0) {
 		DEBUG(DEBUG_ERR, (__location__ " ERROR: num_persistent_updates == 0\n"));
 	} else {
 		client->num_persistent_updates--;
 	}
 
-	DEBUG(DEBUG_ERR,(__location__ " Forcing recovery\n"));
+	DEBUG(DEBUG_ERR,(__location__ " An error occurred during transaction on"
+			 " db_id[0x%08x] - forcing recovery\n",
+			 ctdb_db->db_id));
 	client->ctdb->recovery_mode = CTDB_RECOVERY_ACTIVE;
 
 	return 0;
 }
 
+/**
+ * Tell whether a transaction is active on this node on the give DB.
+ */
+int32_t ctdb_control_trans2_active(struct ctdb_context *ctdb,
+				   struct ctdb_req_control *c,
+				   uint32_t db_id)
+{
+	struct ctdb_db_context *ctdb_db;
+	struct ctdb_client *client = ctdb_reqid_find(ctdb, c->client_id, struct ctdb_client);
+
+	ctdb_db = find_ctdb_db(ctdb, db_id);
+	if (!ctdb_db) {
+		DEBUG(DEBUG_ERR,(__location__ " Unknown db 0x%08x\n", db_id));
+		return -1;
+	}
+
+	if (client->db_id == db_id) {
+		return 0;
+	}
+
+	if (ctdb_db->transaction_active) {
+		return 1;
+	} else {
+		return 0;
+	}
+}
 
 /*
   backwards compatibility:
