@@ -31,7 +31,7 @@ ctdb_test_init "$@"
 
 set -e
 
-onnode 0 $CTDB_TEST_WRAPPER cluster_is_healthy
+cluster_is_healthy
 
 if ! try_command_on_node -v 0 "rpm -q ctdb" ; then
     echo "No useful output from rpm, SKIPPING rest of test".
@@ -48,5 +48,3 @@ else
     echo "BAD: CTDB version != RPM version"
     testfailures=1
 fi
-
-ctdb_test_exit
