@@ -32,7 +32,7 @@ ctdb_test_init "$@"
 
 set -e
 
-onnode 0 $CTDB_TEST_WRAPPER cluster_is_healthy
+cluster_is_healthy
 
 try_command_on_node -v 0 "$CTDB listvars"
 
@@ -49,7 +49,3 @@ while read var x val ; do
 	exit 1
     fi
 done
-
-testfailures=$?
-
-ctdb_test_exit
