@@ -36,7 +36,10 @@ ctdb_test_init "$@"
 
 set -e
 
-onnode 0 $CTDB_TEST_WRAPPER cluster_is_healthy
+cluster_is_healthy
+
+# Reset configuration
+ctdb_restart_when_done
 
 test_node=1
 
@@ -60,10 +63,4 @@ fi
 
 try_command_on_node -v 0 $CTDB disablemonitor -n $test_node
 
-onnode 0 $CTDB_TEST_WRAPPER wait_until_node_has_status $test_node monoff
-
-echo "That worked!  Restarting cluster to restore configuration..."
-
-restart_ctdb
-
-ctdb_test_exit
+wait_until_node_has_status $test_node monoff
